package org.appfuse.web;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.appfuse.model.User;
import org.appfuse.service.UserManager;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.validation.BindException;
import org.springframework.web.bind.RequestUtils;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.NumberFormat;


public class UserFormController extends SimpleFormController {
    private static Log log = LogFactory.getLog(UserFormController.class);
    private UserManager userManager = null;

    public void setUserManager(UserManager userManager) {
        this.userManager = userManager;
    }

    public UserFormController() {
        super();
        setCommandClass(User.class);
    }

    protected Object formBackingObject(HttpServletRequest request)
    throws ServletException {
        String id = RequestUtils.getStringParameter(request, "id", "");
        User command = new User();

        if (!"".equals(id)) {
            command = userManager.getUser(id);
        }

        return command;
    }

    protected void initBinder(HttpServletRequest request,
                              ServletRequestDataBinder binder) {
        // convert java.lang.Long
        NumberFormat nf = NumberFormat.getNumberInstance();
        binder.registerCustomEditor(Long.class,
                                    new CustomNumberEditor(Long.class, nf, true));
    }

    /**
     * Redirect to the successView when the cancel button has been pressed.
     */
    public ModelAndView processFormSubmission(HttpServletRequest request,
                                              HttpServletResponse response,
                                              Object command,
                                              BindException errors)
    throws Exception {
        if (request.getParameter("cancel") != null) {
            return new ModelAndView(getSuccessView());
        }

        return super.processFormSubmission(request, response, command, errors);
    }

    public ModelAndView onSubmit(HttpServletRequest request,
                                 HttpServletResponse response, Object command,
                                 BindException errors)
    throws Exception {
        if (log.isDebugEnabled()) {
            log.debug("entering 'onSubmit' method...");
        }

        User user = (User) command;

        if (request.getParameter("delete") != null) {
            userManager.removeUser(user.getId().toString());
            request.getSession().setAttribute("message",
                                              getMessageSourceAccessor()
                                                  .getMessage("user.deleted",
                                                              new Object[] {
                                                                  user.getFirstName() +
                                                                  ' ' +
                                                                  user.getLastName()
                                                              }));
        } else {
            userManager.saveUser(user);

            // put success message into session so it survives redirects
            request.getSession().setAttribute("message",
                                              getMessageSourceAccessor()
                                                  .getMessage("user.saved",
                                                              new Object[] {
                                                                  user.getFirstName() +
                                                                  ' ' +
                                                                  user.getLastName()
                                                              }));
        }

        return new ModelAndView(getSuccessView());
    }
}
